// 
// Module: CalPNResponse
// Author: Hassan Siddiqui
//
// Description: 
// Provides a clean interface to CAL response data.
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: CalPNResponse.cc,v 1.2 2000/11/01 09:16:58 hsiddiqu Exp $

/* General */
#include <string>
#include <iostream>
#include <vector>

/* SAS-specific */
#include "Dal.h"
#include "CalServer.h"
#include "Redistribution.h"
#include "Tracer.h"

// Local classes
#include "CalPNResponse.h"

/*****************************
 *
 * class CalPNResponse
 *
 * Interface to CAL response data.
 *
 *****************************/


inline void CalPNResponse::init( PixCoords cenRaw )
{
  
  PROC;

  /* Currently set state for default EPN settings */
  // calServer.state()->node()->set(Xmm::PRIMARY_NODE);
  
  /* Get response information */
  CanonicalRedist * rmf = 0; rmf = calServer.getAtom(rmf);

  /* Set source centre in raw coordinates */
  rmf->at(cenRaw.x(), cenRaw.y());

  /* Set PN line number */
  errstr 
    << "CalPNResponse: response model appropriate for line " << cenRaw.y()
    << Sas::message(Msg::App, Msg::Verbose);

  rmf->at(cenRaw.y());   // extract redist spectrum appl. at CCD line

  _rmf = rmf;
  
  ENDPROC;
}

/* Constructors */
/* Provides the entire range of 
 * redistribution available from the CAL */
CalPNResponse::CalPNResponse ( PixCoords cenRaw,
			       unsigned lowerPI,
			       unsigned nOutputChannels,
			       unsigned outputChannelWidth,
                               EventGrades eventGrade)
{
  init(cenRaw);

  _eventGrade=eventGrade;

  createWithCalEbins(lowerPI, 
		     nOutputChannels, 
		     outputChannelWidth);
}  

/* Provides a restricted range */
CalPNResponse::CalPNResponse ( PixCoords cenRaw,
			       unsigned lowerPI,
			       unsigned nOutputChannels,
			       unsigned outputChannelWidth,
			       const BinnedAxis * ebins,
                               EventGrades eventGrade)
{
  init(cenRaw);

  _eventGrade=eventGrade;

   createWithUserEbins( lowerPI, 
		       nOutputChannels, 
		       outputChannelWidth,
		       ebins);
}

/* Destructor */
CalPNResponse::~CalPNResponse()
{
  PROC;
  delete _eboundsAxis; _eboundsAxis = 0;
  ENDPROC;
}

