// Module: LowMemFixedMatrixColumn
// Author: Hassan Siddiqui
//
// Description:
// Fixed length vector column representation of MatrixColumn, for the DAL low memory model
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: LowMemFixedMatrixColumn.h,v 1.1 2000/03/07 13:49:43 hsiddiqu Exp $

#ifndef LowMemFixedMatrixColumn_H
#define LowMemFixedMatrixColumn_H

/* General */

/* SAS-specific */
#include "Dal.h"

/* Local */
#include "MatrixColumn.h"

/*******************************
 *
 * class LowMemFixedMatrixColumn
 *
 *******************************/

class LowMemFixedMatrixColumn : public MatrixColumn
{

public:

  /* Constructor */
  LowMemFixedMatrixColumn( Table * table,
		const vector<unsigned long>& sizeVect);

  /* Destructor */
  ~LowMemFixedMatrixColumn();

  /* Set state to a particular row, and the size of that row */
   void row( unsigned long row,
	     unsigned int elements ); 

  /* Write to a given position in row */
   void writeTo( unsigned int element,
		 double value) ;

  /* The number of elements in row */
   unsigned int rowElements() const ;

  /* Inform class that processing for current row is complete */
   void closeRow();

private:

  Column     * _matrixCol;

  ColumnData * _matrixColData;

  real32     * _matrixData;
  
  unsigned long _row;
  unsigned int _rowelements;

  /* Keep trace of element furthest along row that has been written to. When row is closed, the row will be padded with zeros, from the next element on, to _rowelements. */
  unsigned int _largestelement;

};

#endif
